
import client from "./src/client";
import ADD_EVEMT from "./src/graphql/addEvent";
import getGeoInfo from "./src/events/getGeoInfo";
import sessionStart from "./src/events/startSession";
import paegView from "./src/events/pageView";
import GET_PROJECTID_BY_ORIGIN from "./src/graphql/getProjectIdByOrigin";
import * as PusherPushNotifications from "@pusher/push-notifications-web";
import getVisitorId from "./src/refactor";
import { oneSession } from "./src/events/defaultData";
import UNIFY_DATA from "./src/graphql/UNIFY_DATA";
import { detectScroll } from "./src/events/scroll";


const beamsClient = new PusherPushNotifications.Client({
  instanceId: 'd0893de3-7453-4aa2-88d9-134f8ce3ddfa',
});

const beamsTokenProvider = new PusherPushNotifications.TokenProvider({
  // url: "https://api.freeanalytics.one/pusher/beams-auth",
  url: "https://api.freeanalytics.one/pusher/beams-auth",
  headers: {
    credentials: "include", // Headers your auth endpoint needs
  },
});

export async function startApp() {
  console.log("Started tracking");

  try {

    const projectIdRes = await client.query({
      query: GET_PROJECTID_BY_ORIGIN,
      variables: {
        origin: window.location.origin,
      },
    });

    console.log(projectIdRes);

    const projectId = parseInt(projectIdRes.data.getProjectIdByOrigin)
    console.log("projectId", projectId);
    detectScroll(projectId)

    // endSession(projectId)
    setInterval(() => {
      sessionStart(projectId);
    }, oneSession / 10)

    var previousUrl = "";
    var observer = new MutationObserver(function (mutations) {
      console.log("alive");
      if (window.location.href !== previousUrl) {
        previousUrl = window.location.href;
        console.log(`URL changed to ${window.location.href}`);
        paegView(projectId);
      }
    });

    const config = { subtree: true, childList: true };
    observer.observe(document, config);

    const userAgent = window.navigator.userAgent
    // console.log("browser", browser)

    // Pusher - settings notification

    // beamsClient.clearAllState().then(() => console.log('Beams state has been cleared')).catch(e => console.error('Could not clear Beams state', e));
    // beamsClient.start()
    //     .then(() => beamsClient.addDeviceInterest('hello'))
    //     .then(() => console.log('Successfully registered and subscribed!'))
    //     .catch(console.error);
    const visitorId = await getVisitorId();

    console.log(visitorId + '@' + projectId.toString())
    beamsClient
      .start()
      .then(() => beamsClient.setUserId(visitorId + '@' + projectId.toString(), beamsTokenProvider))
      .catch((error) => {
        // Error first time then clear state
        console.log("error msg:", error.message)
        if (error.message === 'Changing the `userId` is not allowed.' || error.message === 'Unexpected status code 400: Bad request, A user Id is already associated with this device.') {
          beamsClient.clearAllState()
            .then(() => console.log('Beams state has been cleared')).then(() => {
              beamsClient
                .start()
                .then(() => beamsClient.setUserId(visitorId + '@' + projectId.toString(), beamsTokenProvider))
                .catch((error) => {
                  // Error second time then clear state
                  console.log("error msg:", error.message)
                  if (error.message === 'Changing the `userId` is not allowed.' || error.message === 'Unexpected status code 400: Bad request, A user Id is already associated with this device.') {
                    beamsClient.clearAllState()
                      .then(() => console.log('Beams state has been cleared')).then(() => {
                        beamsClient
                          .start()
                          .then(() => beamsClient.setUserId(visitorId + '@' + projectId.toString(), beamsTokenProvider))
                          .catch((error) => {
                            console.log("error msg:", error.message)
                          })
                      }
                      ).catch(e => console.error('Could not clear Beams state', e));
                  }
                })
            }
            ).catch(e => console.error('Could not clear Beams state', e));
        }
      });
  } catch (error) {
    console.log("Project not found", error)
  }

};

export async function unifyUserData(variables: { userProjectId: string, email?: string, displayName?: string, phoneNumber?: string, photoUrl?: string, customInfo?: JSON }) {
  try {
    console.log("unifing user data")
    const visitorId = await getVisitorId();
    const projectIdRes = await client.query({
      query: GET_PROJECTID_BY_ORIGIN,
      variables: {
        origin: window.location.origin,
      },
    });
    const { userProjectId, email, customInfo, displayName, phoneNumber, photoUrl } = variables
    const projectId = parseInt(projectIdRes.data.getProjectIdByOrigin)
    console.log("unfiyDataMutate", visitorId, projectId, userProjectId, email, customInfo)




    console.log(projectIdRes);


    const unfiyDataMutate = await client.mutate({
      mutation: UNIFY_DATA,
      variables: {
        userId: visitorId,
        projectId: projectId,
        userProjectId,
        email,
        customInfo,
        displayName,
        phoneNumber,
        photoUrl: photoUrl,
      }
    })

    console.log("unfiyDataMutateData", unfiyDataMutate)
  } catch (error) {
    // console.log(JSON.stringify(error, Object.getOwnPropertyNames(error)));
    // console.log(error);
    console.log("error:", JSON.stringify(error, null, 2));

  }

}
// unifyUserData({
//   userProjectId: '156115sdfsdfdsf',
//   displayName: "sdf",
//   phoneNumber: "0619",
//   photoUrl:"14555",
// })



// startApp()